/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.jcg.s3proxy;

/**
 *
 * @author omozegieaziegbe
 */
import org.gaul.s3proxy.S3Proxy;
import org.eclipse.jetty.util.component.AbstractLifeCycle;
import org.jclouds.blobstore.BlobStore;
import org.jclouds.blobstore.BlobStoreContext;
import org.jclouds.ContextBuilder;
import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.core.sync.RequestBody;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.model.*;
import java.net.URI;
import java.util.Properties;
import software.amazon.awssdk.core.sync.ResponseTransformer;
import java.util.List;
import org.jclouds.filesystem.reference.FilesystemConstants;

public class S3ProxyExample {

    public static void main(String[] args) throws Exception {
        // Configure the backend BlobStore
        Properties properties = new Properties();
        properties.setProperty(FilesystemConstants.PROPERTY_BASEDIR, "./custom-directory/s3proxy");

        BlobStoreContext context = ContextBuilder
                .newBuilder("filesystem")
                .credentials("identity", "credential")
                .overrides(properties)
                .build(BlobStoreContext.class);
        BlobStore blobStore = context.getBlobStore();

        // Configure and start S3Proxy
        S3Proxy s3Proxy = S3Proxy.builder()
                .blobStore(blobStore)
                .endpoint(URI.create("http://127.0.0.1:8080"))
                .build();
        s3Proxy.start();

        // Wait for S3Proxy to start
        while (!s3Proxy.getState().equals(AbstractLifeCycle.STARTED)) {
            Thread.sleep(100);
        }
        System.out.println("S3Proxy started at: " + s3Proxy.getState());

        // Configure S3Client
        S3Client s3Client = S3Client.builder()
                .endpointOverride(URI.create("http://127.0.0.1:8080"))
                .region(Region.US_EAST_1)
                .credentialsProvider(StaticCredentialsProvider.create(
                        AwsBasicCredentials.create("access", "secret")))
                .build();

        // Perform S3 Operations
        String bucketName = "test-bucket";
        s3Client.createBucket(CreateBucketRequest.builder().bucket(bucketName).build());
        System.out.println("Bucket created: " + bucketName);

        String key = "example.txt";
        String content = "Hello, S3Proxy!";
        s3Client.putObject(PutObjectRequest.builder()
                .bucket(bucketName)
                .key(key)
                .build(),
                RequestBody.fromBytes(content.getBytes()));
        System.out.println("File uploaded: " + key);

        // Verify the upload
        List<S3Object> summaries = s3Client.listObjects(request
                -> request.bucket(bucketName)
        ).contents();
        summaries.forEach(summary -> System.out.println("File in bucket: " + summary.key()));

        String downloadedContent = s3Client.getObject(
                GetObjectRequest.builder()
                        .bucket(bucketName)
                        .key(key)
                        .build(),
                ResponseTransformer.toBytes()).asUtf8String();
        System.out.println("Downloaded content: " + downloadedContent);

        // Shutdown the proxy
        s3Proxy.stop();
        System.out.println("S3Proxy stopped.");
    }
}
